indexing

	description:

		"Eiffel dynamic type sets"

	library: "Gobo Eiffel Tools Library"
	copyright: "Copyright (c) 2004, Eric Bezault and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2004/08/08 09:58:10 $"
	revision: "$Revision: 1.5 $"

deferred class ET_DYNAMIC_TYPE_SET

inherit

	ET_DYNAMIC_TARGET

feature -- Status report

	has_type (a_type: ET_DYNAMIC_TYPE): BOOLEAN is
			-- Does current type set contain `a_type'?
		require
			a_type_not_void: a_type /= Void
		do
			Result := (first_type = a_type)
			if not Result then
				if other_types /= Void then
					Result := other_types.has (a_type)
				end
			end
		end

	is_expanded: BOOLEAN is
			-- Is `static_type' expanded?
		do
			Result := static_type.is_expanded
		ensure
			definition: Result = static_type.is_expanded
		end

feature -- Access

	static_type: ET_DYNAMIC_TYPE is
			-- Type at compilation time
		deferred
		ensure
			static_type_not_void: Result /= Void
		end

	first_type: ET_DYNAMIC_TYPE is
			-- First type in current set;
			-- Void if no type in the set
		deferred
		end

	other_types: ET_DYNAMIC_TYPE_LIST is
			-- Other types in current set;
			-- Void if zero or one type in the set
		deferred
		end

	sources: ET_DYNAMIC_ATTACHMENT is
			-- Sub-sets of current set
		deferred
		end

feature -- Measurement

	count: INTEGER is
			-- Number of types in current type set
		do
			if first_type /= Void then
				if other_types /= Void then
					Result := other_types.count + 1
				else
					Result := 1
				end
			end
		ensure
			count_non_negative: Result >= 0
		end

feature -- Element change

	put_target (a_target: ET_DYNAMIC_TARGET; a_system: ET_SYSTEM) is
			-- Add `a_target' to current set.
			-- (Targets are supersets of current set.)
		require
			a_target_not_void: a_target /= Void
			a_system_not_void: a_system /= Void
		deferred
		end

	put_source (a_source: ET_DYNAMIC_ATTACHMENT; a_system: ET_SYSTEM) is
			-- Add `a_source' to current set.
			-- (Sources are subsets of current set.)
		require
			a_source_not_void: a_source /= Void
			a_system_not_void: a_system /= Void
		deferred
		end

	propagate_types (a_system: ET_SYSTEM) is
			-- Propagate types from `sources'.
		require
			a_system_not_void: a_system /= Void
		local
			l_source: ET_DYNAMIC_ATTACHMENT
		do
			from
				l_source := sources
			until
				l_source = Void
			loop
				l_source.propagate_types (Current, a_system)
				l_source := l_source.next_attachment
			end
		end

end
